from PyInquirer import prompt
from dotenv import load_dotenv
from datetime import datetime
import click
from os.path import expanduser
import os, sys, subprocess, json, logging
from bhav.util import today, fetch, unzip, writeCSV, readCSV, dateList

home = expanduser("~")
config_dir = home + '/.bhav'
config_file = config_dir + '/config'
config = dict()

# read configuration if present
if os.path.exists(config_file):
    with open(config_file, 'r') as handle:
        config = json.loads(handle.read())

@click.group()
def cli():
    pass

@click.group()
def conf():
    pass

# @click.group()
# def chart():
#     pass

# @click.command()
# def start():
#     from charts.chart import Chart
#     chart = Chart()
#     chart.start()

@click.command()
def ls():
    if os.path.exists(config_file):
        with open(config_file, 'r') as handle:
            config = json.loads(handle.read())
            for key in config:
                print(key, ":", config[key])
    else:
        print('No configuration exists')

@click.command()
@click.option('-o', '--output-dir', default=None, help='e.g. C:/mydir . This is where the stock price data (bhavcopy) will be stored')
@click.option('-b', '--bse-url', default=None, help='BSE bhavcopy url template. Default- http://www.bseindia.com/download/BhavCopy/Equity/eq#-#day#-##-#month#-##-#year#-#_csv.zip')
@click.option('-B', '--bse-columns', default=None, help='BSE bhavcopy column order, default - SYMBOL,NAME,SC_GROUP,SC_TYPE,OPEN,HIGH,LOW,CLOSE,LAST,PREVCLOSE,NO_TRADES,VOLUME,NET_TURNOV,TDCLOINDI')
@click.option('-n', '--nse-url', default=None, help='NSE bhavcopy url template. Default - https://www.nseindia.com/content/historical/EQUITIES/#-#year#-#/#n#month#n#/cm#-#day#-##n#month#n##-#year#-#bhav.csv.zip')
@click.option('-N', '--nse-columns', default=None, help='NSE bhavcopy column order, default - SYMBOL,SERIES,OPEN,HIGH,LOW,CLOSE,LAST,PREVCLOSE,VOLUME,TOTTRDVAL,DATE,TOTALTRADES,ISIN')
def update(output_dir, bse_url, bse_columns, nse_url, nse_columns):
    if not os.path.exists(config_file):
        # creating config directory if absent
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)

    # set output directory
    if output_dir is not None:
        config['output_dir'] = output_dir
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
    elif 'output_dir' not in config:
        print("Setting output directory : ", home+ '/bhav')
        if not os.path.exists(home+ '/bhav'):
            os.makedirs(home+ '/bhav')
        config['output_dir'] = home+ '/bhav'
    
    # set bse bhavcopy url format
    if bse_url is not None:
        config['bse_url'] = bse_url
    elif 'bse_url' not in config:
        # https://www.bseindia.com/download/BhavCopy/Equity/EQ290520_CSV.ZIP
        config['bse_url'] = 'http://www.bseindia.com/download/BhavCopy/Equity/EQ#-#day#-##-#month#-##-#YY#-#_CSV.zip'
    
    # set bse bhavcopy url format
    if nse_url is not None:
        config['nse_url'] = nse_url
    elif 'nse_url' not in config:
        config['nse_url'] = 'https://archive.nseindia.com/content/historical/EQUITIES/#-#year#-#/#n#month#n#/cm#-#day#-##n#month#n##-#year#-#bhav.csv.zip'

    # set bse bhavcopy column order
    if bse_columns is not None:
        config['bse_columns'] = bse_columns
    elif 'bse_columns' not in config:
        config['bse_columns'] = 'SYMBOL,NAME,,,OPEN,HIGH,LOW,CLOSE,LAST,PREVCLOSE,,VOLUME,,'
    
    # set nse bhavcopy column order
    if nse_columns is not None:
        config['nse_columns'] = nse_columns
    elif 'nse_columns' not in config:
        config['nse_columns'] = 'SYMBOL,,OPEN,HIGH,LOW,CLOSE,LAST,PREVCLOSE,VOLUME,,DATE,,'
    
    # write config file
    with open(config_file, 'w') as fp:
        json.dump(config, fp)

@click.command()
@click.argument('date', default=today())
@click.option('-s', '--start-date', default=None, help='start date if downloading for a range of dates')
@click.option('-e', '--end-date', default=None, help='end date if downloading for a range of dates')
@click.option('-z', '--zipped', is_flag=True, help='if unzip required')
def bse(date, zipped, start_date, end_date):
    if not start_date:
        dates = [date]
    else:
        if not end_date:
            end_date = today()
        dates = dateList(start_date, end_date)
    
    print("fetching for dates : %s", dates)

    for process_date in dates:
        if not zipped:
            print("fetching uncompressed BSE bhavcopy for %s" % process_date)
            zipped = False
        else:
            print("fetching compressed BSE bhavcopy for %s" % process_date)
            zipped = True

        bse_root = config['output_dir'] + '/bse'
        if not os.path.exists(bse_root):
            os.mkdir(bse_root)
        target_directory = bse_root+ '/' + process_date[:4]
        if not os.path.exists(target_directory):
            os.mkdir(target_directory)
        fetch(config['bse_url'], process_date, target_directory, zipped, config['bse_columns'])
    

@click.command()
@click.argument('date', default=today())
@click.option('-s', '--start-date', default=None, help='start date if downloading for a range of dates')
@click.option('-e', '--end-date', default=None, help='end date if downloading for a range of dates')
@click.option('-z', '--zipped', is_flag=True, help='if unzip required')
def nse(zipped, date, start_date, end_date):
    if not start_date:
        dates = [date]
    else:
        if not end_date:
            end_date = today()
        dates = dateList(start_date, end_date)
    print("fetching for dates : %s", dates)
    for process_date in dates:
        if not zipped:
            print("fetching uncompressed NSE bhavcopy for %s" % process_date)
            zipped = False
        else:
            print("fetching compressed NSE bhavcopy for %s" % process_date)
            zipped = True
        
        nse_root = config['output_dir'] + '/nse'
        if not os.path.exists(nse_root):
            os.mkdir(nse_root)

        target_directory = nse_root +'/'+ process_date[:4]
        if not os.path.exists(target_directory):
            os.mkdir(target_directory)
        fetch(config['nse_url'], process_date, target_directory, zipped, config['nse_columns'])

conf.add_command(ls)
conf.add_command(update)
cli.add_command(conf)

cli.add_command(bse)
cli.add_command(nse)

# chart.add_command(start)
# cli.add_command(chart)

if __name__ == '__main__':
    cli()


import unittest
from bhav.util import today, fetch, unzip, writeCSV, readCSV, dateList

class TestUtilMethods(unittest.TestCase):
    def test_dateList(self):
        self.assertEqual(dateList('20200520', '20200529'), list(['20200520', '20200521', '20200522', '20200523', '20200524', '20200525', '20200526', '20200527', '20200528', '20200529']))

if __name__ == '__main__':
    unittest.main()


from setuptools import setup, find_packages

setup(
    name='bhav',
    version='1.0.10',
    author="Ajay Sharma",
    author_email="deal.ajay@gmail.com",
    python_requires='>=3',
    packages=find_packages(),
    include_package_data=True,
    url="https://gitlab.com/sw8fbar/bhav",
    description="CLI to download End of day Stock data from BSE and NSE",
    long_description=open("README.md").read(),
    license="LICENSE.txt",
    install_requires=[
        'Click',
        'requests',
        'python-dotenv',
        'PyInquirer',
        'flask',
        'pandas',
        'marshmallow',
        'flask_cors'
    ],
    entry_points={
        'console_scripts': [
            'bhav=bhav.main:cli',
        ]
    },
)


import csv, zipfile, os
import requests
from datetime import datetime, timedelta
from io import TextIOWrapper, StringIO

months = {
    '01': {'name':'JAN', 'days': 31},
    '02': {'name':'FEB', 'days': 29},
    '03': {'name':'MAR', 'days': 31},
    '04': {'name':'APR', 'days': 30},
    '05': {'name':'MAY', 'days': 31},
    '06': {'name':'JUN', 'days': 30},
    '07': {'name':'JUL', 'days': 31},
    '08': {'name':'AUG', 'days': 31},
    '09': {'name':'SEP', 'days': 30},
    '10': {'name':'OCT', 'days': 31},
    '11': {'name':'NOV', 'days': 30},
    '12': {'name':'DEC', 'days': 31},
}

def fetch(url_template, date, target_dir, zipped, columns):
    url = url_from_template(url_template, date)
    if (zipped):
        target_zip = target_dir+'/'+ date+'.zip'
        if not os.path.exists(target_dir+'/csv'):
            os.mkdir(target_dir+'/csv')
        target_csv = target_dir+'/csv/'+ date+'.csv'
    else:
        if not os.path.exists(target_dir+'/csv'):
            os.mkdir(target_dir+'/csv')
        target_zip = target_dir+'/csv'+ date+'.csv'
        target_csv = target_zip

    print("fetching from %s into %s " % (url, target_zip))
        # get request
    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.76 Safari/537.36'}
    try:
        response = requests.get(url, headers=headers, timeout=5)
        response.raise_for_status()
        # write to file
        with open(target_zip, "wb") as file:
            file.write(response.content)
            file.close()
            if zipped:
                data = unzip(target_zip)
                if columns.find('DATE') == -1:
                    writeCSV(data, target_csv, columns, addDate=True, date=date)
                else:
                    writeCSV(data, target_csv, columns)
    except requests.exceptions.HTTPError as errh:
        # print (errh.response.text)
        print ("Http Error:", errh)
    except requests.exceptions.ConnectionError as errc:
        print ("Error Connecting:",errc)
    except requests.exceptions.Timeout as errt:
        print ("Timeout Error:",errt)
    except requests.exceptions.RequestException as err:
        print ("Oops: Something Else",err)


def unzip(source):
    print("unzipping %s" % (source))
    with zipfile.ZipFile(source, "r") as zip_ref:
        for file in zip_ref.namelist():
            print("Extracting %s from zip" % file)
            data = []
            with zip_ref.open(file) as myfile:
                reader = csv.reader(TextIOWrapper(myfile, 'utf-8'))
                for row in reader:
                    data.append(row)
            return data
                        
                

def writeCSV(data, target, header, addDate=False, date=None):
    linecount = 0
    if addDate:
        header = 'DATE,'+ header
    f = StringIO(header)
    reader = csv.reader(f, delimiter=',')
    for row in reader:
        print('Setting headers',row)
        header_row =row
    with open(target, "w") as csv_file:
        csv_writer = csv.writer(
            csv_file, delimiter=",", quotechar='"', quoting=csv.QUOTE_MINIMAL, lineterminator='\n'
        )
        for row in data:
            # print(row)
            if linecount == 0:
                 print('Incoming headers',row)
                 csv_writer.writerow(header_row)
                 linecount+=1
            else:
                if addDate:
                    row.insert(0, date[6:] + '/'+ date[4:6]+'/' + date[:4])
                csv_writer.writerow(row)
                linecount+=1
        csv_file.close()


def readCSV(source):
    data = []
    line_count = 0
    with open(source, "r") as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=",")
        for row in csv_reader:
            if line_count == 0:
                print(f'Column names are {", ".join(row)}')
                line_count += 1
            else:
                line_count += 1
                data.append(row)
    return data

def today():
    return datetime.today().strftime('%Y%m%d')

def url_from_template(template, date):
    template = template.replace('#-#day#-#', date[6:])
    template = template.replace('#-#month#-#', date[4:6])
    template = template.replace('#n#month#n#', months[date[4:6]]['name'])
    template = template.replace('#-#year#-#', date[:4])
    template = template.replace('#-#YY#-#', date[2:4])
    return template

def dateList(start, end):
    start = datetime.strptime(start, "%Y%m%d")
    end = datetime.strptime(end, "%Y%m%d")
    date_generated = [start + timedelta(days=x) for x in range(0, (end-start).days + 1)]
    dates = [x.strftime('%Y%m%d') for x in date_generated]

    return dates
